#pragma once

#include <wlr/types/wlr_compositor.h>
#include <wlr/types/wlr_scene.h>
#include <wlr/types/wlr_input_method_v2.h>
#include <wlr/types/wlr_text_input_v3.h>

struct hikari_input_method_relay {
  struct wlr_seat *seat;

  struct wl_list text_inputs;
  struct wl_list input_popups;
  struct wlr_input_method_v2 *input_method;

  struct wl_listener text_input_new;

  struct wl_listener input_method_new;
  struct wl_listener input_method_commit;
  struct wl_listener input_method_new_popup_surface;
  struct wl_listener input_method_grab_keyboard;
  struct wl_listener input_method_destroy;

  struct wl_listener input_method_keyboard_grab_destroy;
};

struct hikari_text_input {
  struct hikari_input_method_relay *relay;

  struct wlr_text_input_v3 *input;

  struct wlr_surface *pending_focused_surface;

  struct wl_list link;

  struct wl_listener pending_focused_surface_destroy;

  struct wl_listener text_input_enable;
  struct wl_listener text_input_commit;
  struct wl_listener text_input_disable;
  struct wl_listener text_input_destroy;
};

void
hikari_input_method_relay_init(
    struct wlr_seat *seat, struct hikari_input_method_relay *relay);

void
hikari_input_method_relay_fini(struct hikari_input_method_relay *relay);

void
hikari_input_method_relay_set_focus(
    struct hikari_input_method_relay *relay, struct wlr_surface *surface);

struct hikari_text_input *
hikari_text_input_create(struct hikari_input_method_relay *relay,
    struct wlr_text_input_v3 *text_input);

struct hikari_input_popup {
  struct hikari_input_method_relay *relay;

  struct wlr_scene_tree *scene_tree;
  struct wlr_scene_node *relative;
  struct hikari_view *view;

  struct wlr_input_popup_surface_v2 *popup_surface;
  struct wlr_output *fixed_output;

  struct wl_list link;

  struct wl_listener popup_destroy;
  struct wl_listener popup_surface_commit;
  struct wl_listener popup_surface_map;
  struct wl_listener popup_surface_unmap;

  struct wl_listener focused_surface_unmap;
};
