#include <hikari/mode_config.h>

#include <errno.h>
#include <string.h>

void
hikari_mode_config_init(struct hikari_mode_config *mode_config)
{
  mode_config->width = 0;
  mode_config->height = 0;
  mode_config->refresh_rate = 0.f;
  mode_config->custom = false;
}

void
hikari_mode_config_set(
    struct hikari_mode_config *mode_config, int w, int h, float r, bool custom)
{
  mode_config->width = w;
  mode_config->height = h;
  mode_config->refresh_rate = r;
  mode_config->custom = custom;
}

bool
hikari_mode_config_parse(
    struct hikari_mode_config *mode_config, const ucl_object_t *mode_config_obj)
{
  bool success = false;
  int width, height;
  const char *mode_name;
  char *wend, *hend, *hz;
  float refresh_rate = 0.f;

  if (!ucl_object_tostring_safe(mode_config_obj, &mode_name)) {
    fprintf(
        stderr, "configuration error: expected string for container mode\n");
    goto done;
  }
  wend = strchr(mode_name, 'x');
  hend = strchr(mode_name, '@');
  hz = strcasestr(mode_name, "hz");
  if (wend == NULL)
    goto parse_done;
  width = strtol(mode_name, &wend, 10);
  if (width == 0 && errno == EINVAL)
    goto parse_done;
  wend++;
  height = strtol(wend, &hend, 10);
  if (height == 0 && errno == EINVAL)
    goto parse_done;
  if (hend != NULL) {
    hend++;
    refresh_rate = strtof(hend, &hz);
    if (refresh_rate == 0.0 && errno == ERANGE)
      goto parse_done;
  }
  hikari_mode_config_set(mode_config, width, height, refresh_rate, false);
  success = true;

parse_done:
  if (!success)
    fprintf(stderr, "modeline \"%s\" could not be parsed\n", mode_name);

done:
  return success;
}
